@@ -1,8 +1,8 @@
 #include <emsg: see ... '~$ sh aaa.sh.c -h'   (other opt:no/-m/-w/)>	/*
 C='^[#]SH_'			;O=${0##*[/]};R=`dirname $0`/;R0=$R$O;Re=eval\ ;R=$R${O%%.*}
 O=${0##*.};Rs=$R.$O;Rm=$R.tmp.$O;Rh=$R.h;R=$Rs$Rh$Rm;Rp='printf %s\n ';Rc=:;O="
-";[ "${R##*$R0*}" = '' ]&&$Rp"$0:NGext"&&exit 1;R='sed -ne ';Cm=$R'"/[E]ND/!d;:l
-n;p;b l"<$R0>$Rm;$Rp"$Rm"';Rw=$R'"/$C$R/!d;:l;n;/${C}ED/q;p;b l"<$R0';Cw="(R=LS
+";[ -z "${R##*$R0*}" ]&&$Rp"$0:NGext"&&exit 1;R='sed -ne ';Cm=$R'"/[E]ND/!d;: l
+n;p;b l"<$R0>$Rm;$Rp"$Rm"';Rw=$R'"/$C$R/!d;: l;n;/${C}ED/q;p;b l"<$R0';Cw="(R=LS
 $Rw;$Rw>&3;R=HD;$Rw;R=SC;$Rw>&3)"'>$Rh 3>$Rs;$Rp"$Rh $Rs"';RB=$($R"s/${C}OP//p"\
 <$R0|(F=mw;while read -r a b;do B=${a%:};F=`$Rp"$F"|$R"s#$B:*##;p"`${a%_};$Rp"
 C$B=\$(cat<<'E'$O$b${O}E$O)";done;$Rp"R1=$F"));$Re"$RB";while getopts $R1 R;do
@@ -11,7 +11,7 @@ case $R in \?)exit 1;;*)$Re"O$R=\$OPTARG";Rc=$Rc$O`$Re'$Rp"$C'$R\"`;;esac;done
 
 #SH_LS
 #!/bin/sh
-cat << 'EEE' > /dev/null
+:<< 'E==='
 /* Copyright (C) 2020 Momi-g
  *
  * This program is free software; you can redistribute it and/or modify
@@ -27,9 +27,9 @@ cat << 'EEE' > /dev/null
  * You should have received a copy of the GNU General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
-EEE
+E===
 
-cat << 'EEE' > /dev/null
+: << 'E==='
 #SH_doc
 title=*SH_bn* section=1 repnl=\040
 @name *SH_bn*
@@ -43,36 +43,37 @@ title=*SH_bn* section=1 repnl=\040
 	~$ sh src.sh.c -a -b -a		#>>hw, exec ls, hw
 		@()
 @_opt
-	@(list_o)
+		@(list_o)
 	-hHV: usage, version
 	-Q: output corecode
 	-sS: output sample
-	@()
+		@()
 @_desc
 	*SH_bn* is portable preprocesser. code is separated with 2 parts,
 	main code(#inc...#END) and option code (//SH_OP ...). workflow is:
-	@(code)--@
-	  1: exec src as shell script. ~$ sh src.sh.c -a
-	  2: main code searches regex suitable line (dfl: //SH_OP, /*SH_OP etc)
-	  3: read hitline and save string to vars ($Ca, $Cb, $C... etc)
+		@(code)--@
+	  1: exec src as shell script. eg) ~$ sh src.sh.c -a
+	  2: header code searches regex suitable line (dfl: //SH_OP, /*SH_OP etc)
+	  3: read hitline and save as cmd string ($Ca, $Cb, $C... etc)
 	  4: exec option code if you use -a >> eval "$Ca", -z >> eval "$Cz"
-	@()--@
+		@()--@
 
 	@(raw).SS main code@()
 	--
-	brp exec the head 10 line code as shell script and exit.
-	@(code)--
+	brp exec the head 10 line code as shell script and exit. --
+	linetop '#include...' is assert() for c/cpp (~$ cc src.sh.c #>>stop)
+	so you can delete if you want.
+		@(code)--
 	#inc...
 	C='^[/][/*]SH_'    ; ...
 		...#END
-	@()--
-	main code holds regex string var `C` to get option code line. this var is
+		@()--
+	main code holds regex expression var `C` to get option code line. this var is
 	used as:
-	@(code)--
+		@(code)--
 	~$ cat src.sh.c | sed -ne "/$C/p"		#BRE-reg. 
-	@()--
-	and gather //SH_OP or //SH_??? etc. linetop '#include...' is assert()
-	for c/cpp (~$ cc src.sh.c #stop) so you can delete if you want.	--
+		@()--
+	and gather //SH_OP or //SH_??? etc. 	--
 	main code holds predefined option code, -w, -m. see below for details.
 	--
 	@(raw).SS option code@()
@@ -84,16 +85,16 @@ title=*SH_bn* section=1 repnl=\040
 	@()
 	@(list)
 	1: dfl opthead is "//SH_" or "/*SH_". see below '$C'.
-	2: option line suffix `OP` is fixed. 
+	2: option word suffix `OP` is fixed. 
 	3: separate with blank char(spaces or tabs)
 	4: optchar is [a-zA-Z0-9] or `_`(see below). add colon if use optarg
-	5: write raw sh-script with 'ONE LINE'. code is used as below.
+	5: write raw sh-script with 'ONE LINE'. code is used as the below.
 		@(code)@
 		Cb=$(cat<<'E'
 		z=100;echo "good-bye $2 $Ob $1 $z $R0"
 		E
 		)
-		eval "$Cb"		#if -b set
+		eval "$Cb"		#if exec: ~$ src.sh.c -b
 		@()
 	-: if you set special optchar `_`, it works only once at the script
 		beginning. --
@@ -111,9 +112,11 @@ title=*SH_bn* section=1 repnl=\040
 
 	@(raw).SS about reserved vars or other@()--
 	1-2char var names R?, C?, O? are reserved (R,C,O,Ra,R7,Cg,O5 ...).
-	optarg is set to $O? ( a: >> $Oa ,  v: >> $Ov etc)
+	optarg is set to $O? ( a: >> $Oa ,  v: >> $Ov etc) --
+	eg) //SH_OP a: echo "$Oa" #>> ~$ sh src.sh.c -a1 -a 5 #>> disp 1, 5
+	
 	@(list)
-	$R0,Rm,Rs,Rh: orig/tMp/src/header fname. ($Rm -> src.tmp.c etc)
+	$R0,Rm,Rs,Rh: orig/tMp/src/header fname. ($Rm == src.tmp.c etc)
 		fname uses topname + last suffix. eg) src.xx.yy.py3 -> ./src.py3 --
 		avoid fname unnecessary conflict. --
 		(sh src.c -w >> src.h / src.c ...orig destroyed)
@@ -185,7 +188,7 @@ title=*SH_bn* section=1 repnl=\040
 	sed command pseudocode is the follows:
 	@(code)--@
 	------
-	sed -ne '/[E]ND/{n;b l};d;:l;p;n;b l'
+	sed -ne '/[E]ND/{n;b l};d;: l;p;n;b l'
 	>>
 	sed -n(o print. print only when requested) -e(xpression as script)
 	
@@ -194,7 +197,7 @@ title=*SH_bn* section=1 repnl=\040
 	  goto label l ...b l (b=jump/goto. b abc -> goto abc, needs space@posix)
 	}
 	del line (& read nextline & *goto top*) ...d (d cmd is hard worker)
-	label l:		... :l (label. ':' + 'lbl name') 
+	label l:		... : l (label. ':' + '1sp' + 'lbl name') 
 	p(rint line)		... p
 	n(extline read)		... n
 	goto label l		... b l
@@ -253,16 +256,13 @@ title=*SH_bn* section=1 repnl=\040
 	- others ... see unix philosophy.
 		@()
 @conforming_to	posix-shell
-@copyright
-	Copyright (C) 2020 Momi-g --
-	License GPLv3+ <https://gnu.org/licenses/gpl.html>
-@_ver 2022-03-26 v1.0.3 (2020-01-04  v1.0.0)
+@copyright Copyright (C) 2020 Momi-g, GPLv3+
+@_ver 2023-07-11 v1.0.4 (2020-01-04  v1.0.0)
 @_see
 https://en.wikipedia.org/wiki/Glob_%28programming%29	--
 http://catb.org/%7Eesr/writings/taoup/html/ch10s05.html
-
 #SH_docE
-EEE
+E===
 #SH_ED
 
 #SH_HD
@@ -270,7 +270,7 @@ EEE
 
 #SH_SC
 f_usage(){
-cat << 'EEE'
+cat << 'E==='
 HowTo (*SH_bn*, line oriented universal preprocessor using posix-sh)
 opt: -hH(elp), -V(ersion), -Q(corecode), -sS(ample)
 ------
@@ -290,33 +290,32 @@ opt: -hH(elp), -V(ersion), -Q(corecode), -sS(ample)
 
  your extension: add '//SH_OP z echo "12345"' to this file end and run.
 	~$ sh src.sh.c -z -w -z  #>> 12345 x2 ..see also  ~$ brp -H|less
-EEE
+E===
 exit 0
 }
 
-f_usage_H(){
- cat << 'E E'|sed -e'1d;$d'
-	#SH_rf* *SH_bn*.1.txt
-E E
+f_usageH(){
+	cat << 'E==='	#|sed -e'1d;$d'
+	#SH_rf* 0 *SH_bn*.1.txt
+E===
 exit 0
 }
 
-f_version_info(){
-cat << 'EEE'
+f_verinfo(){
+	cat <<- 'E==='
 *SH_bn* *SH_ver*
-Copyright (C) 2020 Momi-g
-License GPLv3+: GNU GPL version 3 or later <https://gnu.org/licenses/gpl.html>
-EEE
+*SH_copy*
+	E===
 exit 0
 }
 
 f_eg1(){
-cat<<-HDOC
+cat<<-E===
 $code
 
 	//SH_OP a echo "hw"
 	//SH_OP b ls -al
-HDOC
+E===
 exit 0
 }
 
@@ -345,27 +344,38 @@ HDOC_
 exit 0
 }
 
-code=$(cat<<'HDOC' | sed -e 's@\[#]@[/][/*]@'
-#SH_rf* 0 *SH_bn*.head
-HDOC
+code=$(cat<<'E===' | sed -e 's@\[#]@[/][/*]@'
+#include <emsg: see ... '~$ sh aaa.sh.c -h'   (other opt:no/-m/-w/)>	/*
+C='^[#]SH_'			;O=${0##*[/]};R=`dirname $0`/;R0=$R$O;Re=eval\ ;R=$R${O%%.*}
+O=${0##*.};Rs=$R.$O;Rm=$R.tmp.$O;Rh=$R.h;R=$Rs$Rh$Rm;Rp='printf %s\n ';Rc=:;O="
+";[ -z "${R##*$R0*}" ]&&$Rp"$0:NGext"&&exit 1;R='sed -ne ';Cm=$R'"/[E]ND/!d;: l
+n;p;b l"<$R0>$Rm;$Rp"$Rm"';Rw=$R'"/$C$R/!d;: l;n;/${C}ED/q;p;b l"<$R0';Cw="(R=LS
+$Rw;$Rw>&3;R=HD;$Rw;R=SC;$Rw>&3)"'>$Rh 3>$Rs;$Rp"$Rh $Rs"';RB=$($R"s/${C}OP//p"\
+<$R0|(F=mw;while read -r a b;do B=${a%:};F=`$Rp"$F"|$R"s#$B:*##;p"`${a%_};$Rp"
+C$B=\$(cat<<'E'$O$b${O}E$O)";done;$Rp"R1=$F"));$Re"$RB";while getopts $R1 R;do
+case $R in \?)exit 1;;*)$Re"O$R=\$OPTARG";Rc=$Rc$O`$Re'$Rp"$C'$R\"`;;esac;done
+[ "$Rc" = : ]&&Rc=$Cm;shift $((OPTIND-1));$Re"$C_$O$Rc";exit #END  GPLv3+*/
+E===
 )
-# | sed -e '1d;$d;s@\[#]@[/][/*]@1'
 
-f_err(){ echo "*SH_bn*: err. $*">/dev/stderr;exit 1; }
-*SH_bn*_main()(
-	while getopts ":hHVQsS" c; do case "$c" in				
-	[?:]) f_err "bad opt: $OPTARG, $*" ;;	
-	h) f_usage	;;
-	H) f_usage_H	;;
-	V) f_version_info	;;
-	Q) printf '%s\n' "$code"; exit 0	;;
-	s) f_eg1	;;
-	S) f_eg2	;;
-	esac; done; shift $(($OPTIND-1))
+f_err(){ echo "*SH_bn*: err. $*">/dev/stderr; exit 1; }
+*SH_bn*()(
+	OPTIND=1
+		while getopts ":hHVQsS" c && xx=1; do
+		while [ "$xx" = 1 ]&&xx=0; do case "$c" in	#-- xx: fall-through magic		
+	(h) f_usage	;;
+	(H) f_usageH	;;
+	(V) f_verinfo	;;
+	(Q) printf '%s\n' "$code"; exit 0	;;
+	(s) f_eg1	;;
+	(S) f_eg2	;;
+	([?:]) f_err "bad opt: $OPTARG: $*" ;;	
+		esac; done; done; shift $(($OPTIND-1))
 	
 	#direct use
 	printf '%s\n' "$code"
 )
+*SH_bn*_main(){ *SH_bn* "$@"; }
 *SH_bn*_main "$@" || exit	#SH_MAIN
 
 #SH_TSS
@@ -389,6 +399,10 @@ test_2(){
 cat<<'EEE'>/dev/null
  change log
  --
+2023-07-11	Momi-g	<dmy@dmy.dmy>
+
+	* *SH_bn*(all): fix :l >> : l, improve doc etc. v1.0.4
+
 2022-03-26  Momi-g	<dmy@dmy.dmy>
 
 	* *SH_bn*: fix sed, bl >> b l, posix sed jumpcmd needs space. v1.0.3
@@ -411,28 +425,33 @@ EEE
 #SH_OP _ a=`sed -ne "/${C}DF/!d;:l;n;/${C}DE/q;p;bl"<$R0`;eval "$a";set +e	#*/
 #SH_OP h $p"-tsbS:test/eg/.o/.so -LMP:leak,mem,prof -f:funcs -o:bldout	GPLv3+"	 #*/
 #SH_OP f sed -ne "/${C}DF/q;/;/d;/^[a-z].*)/p"<$R0|tr -d ' \t'|grep -F '()' #*/
-#SH_OP t $e"$CW";ftt "$@";bash --posix $Ox $tf
-#SH_OP x Ox="-x"
+#SH_OP t $e"$CW";ftt "$@";$p"bash --posix $Ox $tf"|fvd
+#SH_OP T $e"$CW";ftt "$@";$p"dash $Ox $tf"|fvd
+#SH_OP i $e"$Cb";$p'shellcheck -s sh -e SC2046,SC2086,SC2004,SC2154,SC2015,SC2016,SC2006,SC2030,SC2031,SC2145 $bn.sh'|fv
+#SH_OP s fgr0 "${C}SMP" "${C}SMPE"<$R0>eg.sh #;$p"luajit eg.lua"|fv
+#SH_OP x Ox="-x";fvd(){ fv 2>&1|tee dbg.txt; } 
+#SH_OP v Ov="-v2"
 
 #SH_OP b $e"$CW"
-#SH_OP W $e"$Cm$O$Cw">/dev/null;fborn;chmod 755 $bn;$p"$bn $Rs $Rh $tf"	#*/
-#SH_OP o $e"$CW";$p'fman $Rh 1'|fv		#*/
+#SH_OP W fpp;$e"$Cm$O$Cw">/dev/null;fborn;chmod 755 $bn $bn.sh;$e"$Cs";$p"$bn $Rs $Rh $tf"	#*/
+#SH_OP d $e"$CW";$p'fman $Rh 1'|fv		#*/
 
 #SH_DF
 #-- noob
 fman()( $p"fgr0 '${C}doc' '${C}docE'<$1|amn >$bn.$2
- mandoc -Thtml <$bn.$2 >$bn.$2.html
- man -Tutf8 -l -<$bn.$2|sed -e 's/.`printf \"\\b\"`//g'>$bn.$2.txt
- "|fv
- #(echo '.mso ja.tmac';cat $bn.$2)|man -Tutf8 -l -|sed -e 's/.`printf \"\\b\"`//g'>$bn.$2.txt"|fv
+mandoc -Thtml <$bn.$2 >$bn.$2.html
+man -Tutf8 -l -<$bn.$2|sed -e 's/.`printf \"\\b\"`//g'>$bn.$2.txt
+"|fv
+#(echo '.mso ja.tmac';cat $bn.$2)|man -Tutf8 -l -|sed -e 's/.`printf \"\\b\"`//g'>$bn.$2.txt"|fv
 )
 
 #-- vars
 bn=`basename ${Rs%.*}`; tf=${Rs%/*}/${bn}.ts.${Rs##*.}; e="eval "; p="$Rp"
 #-- mod
 fv()(while read -r a;do $e"cat<<E$O# $a${O}E"|sed -e 's@-L.*-L[^ ]*@-L(omit)@g'>/dev/stderr;$e"$a";done)
+fvd(){ fv; }
 
-fbn()(sed -e "s@\*${C##*]}bn\*@$bn@g"|frf|frv|flit)
+fbn()(sed -e "s@\*${C##*]}bn\*@$bn@g"|frv|fry|frh|flit|frf)
 fsn()(tr -s ' \t' '\n')
 fsl()(tr -s '\n' ' ')
 fu()(fsn|sort -u)
@@ -445,33 +464,100 @@ fg()(sed -ne "s/.*${C##*]}co\*\([^*]*\).*$/\1/p" "$@"|fsn|awk '!a[$0]{a[$0]=1;pr
 
 #-- longcmd
 frf()(
- # *sh_rf* 0 a.txt b.txt ...でcat纏めて出力 top0でsrcinfoは無し出力
- awk -v tg="${C##*]}rf" -v cs='#' -v ce='' 'index($0,tg){
- s=substr($0, index($0,tg)+length(tg)+1);split(s, a)
- m="[ -f \"%s\" ]&&(echo \"%s--copyfrom %s %s\";cat \"%s\";echo \"%s--copyend %s %s\")"
- mm="[ -f \"%s\" ]&&(_=\"%s%s%s\";cat \"%s\";_=\"%s%s%s\")"
- for(i=1;i in a;i++){v=a[i];if(v==0){m=mm;continue};system(sprintf(m,v,cs,v,ce,v,cs,v,ce))}
- next
- }
- {print}'
+# *sh_rf* 0 a.txt b.txt ...でcat纏めて出力 top0でsrcinfoは無し出力
+awk -v tg="${C##*]}rf" -v cs='#' -v ce='' 'index($0,tg){
+s=substr($0, index($0,tg)+length(tg)+1);split(s, a)
+m="[ -f \"%s\" ]&&(echo \"%s--copyfrom %s %s\";cat \"%s\";echo \"%s--copyend %s %s\")"
+mm="[ -f \"%s\" ]&&(_=\"%s%s%s\";cat \"%s\";_=\"%s%s%s\")"
+for(i=1;i in a;i++){v=a[i];if(v==0){m=mm;continue};system(sprintf(m,v,cs,v,ce,v,cs,v,ce))}
+next
+}
+{print}'
 )
-frv()(buf=`awk '$1=="@_ver" {print $3;exit}'<$R0`;sed -e "s@\*${C##*]}ver\*@$buf@g")
+
+
+
+#-- fam '^@_copy' >> take line rval, 1st hit only
+fam(){
+ awk -v rg="$1" '$1~rg {
+ $1=""
+ sub("^[ \t]*", "", $0)
+ print
+ exit
+ }'
+}
+
+
+frv()(buf=$(fam "@_?ver"<$R0);sed -e "s@\*${C##*]}ver\*@$buf@g")
+fry()(buf=$(fam "^@_?license|^@_?copy"<$R0);sed -e "s@\*${C##*]}copy\*@$buf@g")
+	# frv()(buf=`awk '$1=="@_ver" {print $3;exit}'<$R0`;sed -e "s@\*${C##*]}ver\*@$buf@g")
+	# fry()(buf=`awk '$1~ /^@license|^@copy/ {$1="";print;exit}'<$R0`;sed -e "s@\*${C##*]}copy\*@$buf@g")
+
+#-- replace help impl, cmd系help自動生成. amnから自動取得して加工
+frh_()(
+ src="$1"
+ nm=$($p"$src"| fam "^@_?name")
+ ab=$($p"$src"| fam "^@_?brief")
+	
+	#--get_tldr
+ s='/@tl_dr/!d
+	: l;n; /@[(]code/!b l
+	: L;n; /@[(][)]/q
+	p;b L'
+ tldr=`$p"$src"|sed -ne "$s" `
+ 
+	#--get_syno
+  s='/^[ \t]*@_*syno/!d
+ 	: l;n; /^[ \t]*@_*tl_dr/q
+ 	p;b l'
+
+ opts=$(
+ $p"$src"| sed -ne "$s" |awk '{
+	while(match($0, /\[-[^]]*\]/)) {
+ 	sp=", "
+ 	if(length(buf)+RLENGTH+2 > 78){sp="\n "}
+ 	buf=buf sp substr($0, RSTART+1, RLENGTH-2)
+ 	$0=substr($0, RSTART+RLENGTH)
+	}
+ }
+ END{print buf}'
+ )
+ opts=${opts#??}	#>> del 2char: EOS ', '
+	#--make text
+ s=$(printf 'HowTo (%s, %s)\nopt: %s\n---\n%s' "$nm" "$ab" "$opts" "$tldr" )
+ $p"$s"
+)
+
+frh()(
+ sa=$(cat)
+ rs=$(frh_ "$sa")
+ w="*${C##*]}he*"
+  
+ $p"$sa"| awk -v w="$w" -v rs="$rs" '$1==w {$0=rs};{print}'
+ #-- str書換はawk推奨。s///gだとstrが'/'を含有してると腐る wordなら可能性減
+)
+
 flit()(sed -ne "/${C}lit/bl;p;d;:l;n;/${C}litE/d;"'s/[\]/&&/g;s/"/\\"/g;s@.*@"&\\n"@g;p;bl')
 
 fte()(
  cat > $Rm-
- suite=$(cat $Rm-|awk '$1~/^test_[^(]*\(\)/{print $1}'|sed -e 's/[(].*//g')
+ suite=$(cat $Rm-|awk '$1~/^test_[^(]*\(\)/{print $1}'|sed -e 's/[(].*//g'|$e"$gfn")
  [ $# = 0 ] && suite="$suite $suite"|| suite="$suite $@"
  suite=`$p'%s\n' "$suite"|tr '[ \t]' '\n'|sort|uniq -d|fsl`
- (cat $bn $Rm-|grep -vF "${C##*]}MAIN"
- $p' [ $# = 0 ] && set -- '"$suite"';for fc;do $fc ;done;_res')
+ (cat $Rs $Rm-|grep -vF "${C##*]}MAIN"
+ $p'[ $# = 0 ]&&set -- '"$suite"';for fc;do echo "--$fc():">/dev/stderr;$fc;done;_res')
  rm -f $Rm-
 )
-ftt()(shunit_m>$Rm;fte "$@"<$tf>>$Rm;mv $Rm $tf)
+ftt()(
+ [ ${#1} != 0 ] && [ "${1#[012]}" = '' ] && Ov=-v$1 && shift
+ shunit_m $Ov>$Rm;fte "$@"<$tf>>$Rm;mv $Rm $tf
+)
 
 fborn(){
- fgr0 "${C}TSS" "${C}TSE"<$R0|fbn>$tf
  fgR "${C}TSS" "${C}TSE"<$Rs|fbn|tee $Rm>$bn;mv -f $Rm $Rs;fbn<$Rh>$Rm;mv $Rm $Rh
- fgR "${C}TSS_" "${C}TSE"<$R0 |fgr0 "${C}TSS" "${C}TSE"|fbn|fte>tests.code
+ gfn="grep -v '_$'";fgR "${C}TSS_" "${C}TSE"<$R0 |fgr0 "${C}TSS" "${C}TSE"|fbn|fte>tests.code
+ gfn="cat -";fgr0 "${C}TSS" "${C}TSE"<$R0|fbn>$tf
+ R0="${R0%?}"
 }
+fpp(){ awk "{gsub(/[*]${C##*]}ln[*]/,NR);print}"<$R0>${R0}0; R0="$R0"0; $p"${R0}"; }
 #SH_DE
